@@ -104,7 +104,7 @@ struct Configuration {
 	char *channel_command_prefix;
 	long handshake_data_flood_amount;
 	long handshake_data_flood_ban_time;
-	int handshake_data_flood_ban_action;
+	BanAction *handshake_data_flood_ban_action;
 	struct ChMode modes_on_join;
 	int modes_on_join_set;
 	char *level_on_join;
@@ -125,6 +125,15 @@ struct Configuration {
 	long spamfilter_detectslow_fatal;
 	int spamfilter_stop_on_first_match;
 	int spamfilter_utf8;
+	SpamfilterShowMessageContentOnHit spamfilter_show_message_content_on_hit;
+	char *central_spamfilter_url;
+	char *central_spamfilter_feed;
+	long central_spamfilter_refresh_time;
+	int central_spamfilter_verbose;
+	int central_spamfilter_enabled;
+	SecurityGroup *central_spamfilter_except;
+	BanActionValue central_spamfilter_limit_ban_action;
+	long central_spamfilter_limit_ban_time;
 	int maxbans;
 	int watch_away_notification;
 	int uhnames;
@@ -176,6 +185,13 @@ extern MODVAR Configuration iConf;
 extern MODVAR Configuration tempiConf;
 extern MODVAR int ipv6_disabled;
 
+typedef struct BestPractices BestPractices;
+struct BestPractices {
+	int hashed_passwords;		/**< Use hashed passwords */
+	int hashed_passwords_hits;	/**< How many times advice has been given (like 'warnings' but it is advice) */
+};
+extern MODVAR BestPractices bestpractices;
+
 #define KLINE_ADDRESS			iConf.kline_address
 #define GLINE_ADDRESS			iConf.gline_address
 #define OPER_MODES			iConf.oper_modes
