@@ -334,8 +334,6 @@ void fix_timers(void)
 		/* users */
 		if (MyUser(client))
 		{
-			if (client->local->next_nick_allowed > TStime())
-				client->local->next_nick_allowed = TStime();
 			if (client->local->nexttarget > TStime())
 				client->local->nexttarget = TStime();
 		}
@@ -518,6 +516,7 @@ void SetupEvents(void)
 	EventAdd(NULL, "throttling_check_expire", throttling_check_expire, NULL, 1000, 0);
 	EventAdd(NULL, "memory_log_cleaner", memory_log_cleaner, NULL, 61500, 0);
 	EventAdd(NULL, "detect_high_connection_rate", detect_high_connection_rate, NULL, 1000*DETECT_HIGH_CONNECTION_RATE_SAMPLE_TIME, 0);
+	EventAdd(NULL, "central_spamfilter_download_evt", central_spamfilter_download_evt, NULL, 5000, 0);
 }
 
 /** The main function. This will call SocketLoop() once the server is ready. */
@@ -823,6 +822,7 @@ int InitUnrealIRCd(int argc, char *argv[])
 	default_class->sendq = DEFAULT_RECVQ;
 	default_class->name = "default";
 	AddListItem(default_class, conf_class);
+	free_config_defines();
 	if (config_read_start() < 0)
 		exit(-1);
 	while (!is_config_read_finished())
@@ -857,9 +857,9 @@ int InitUnrealIRCd(int argc, char *argv[])
 #endif
 		exit(9);
 	}
-	unreal_log(ULOG_INFO, "config", "CONFIG_PASSED", NULL, "Configuration test passed OK");
 	if (loop.config_test)
 	{
+		unreal_log(ULOG_INFO, "config", "CONFIG_PASSED", NULL, "Configuration test passed OK");
 		fflush(stderr);
 		exit(0);
 	}
