@@ -35,6 +35,15 @@ ModuleHeader MOD_HEADER
  * Once that happens, best to re-check/audit the source.
  */
 
+/* The first "quick" recheck: */
+static long BLACKLIST_RECHECK_TIME_FIRST = 120;
+
+/* After that, check every <this>: */
+static long BLACKLIST_RECHECK_TIME = 900;
+
+#define LastBLCheck(x)	(moddata_client(x, blacklistrecheck_md).l)
+#define SetLastBLCheck(x, y)	do { moddata_client(x, blacklistrecheck_md).l = y; } while(0)
+
 /* Types */
 
 typedef enum {
@@ -64,10 +73,11 @@ struct Blacklist {
 	char *name;
 	BlacklistBackendType backend_type;
 	BlacklistBackend *backend;
-	int action;
+	BanAction *action;
 	long ban_time;
 	char *reason;
 	SecurityGroup *except;
+	int recheck;
 };
 
 /* Blacklist user struct. In the c-ares DNS reply callback we need to pass
@@ -80,7 +90,7 @@ struct BLUser {
 	int is_ipv6;
 	int refcnt;
 	/* The following save_* fields are used by softbans: */
-	int save_action;
+	BanAction *save_action;
 	long save_tkltime;
 	char *save_opernotice;
 	char *save_reason;
@@ -91,11 +101,14 @@ struct BLUser {
 
 /* Global variables */
 ModDataInfo *blacklist_md = NULL;
+ModDataInfo *blacklistrecheck_md = NULL;
 Blacklist *conf_blacklist = NULL;
 
 /* Forward declarations */
 int blacklist_config_test(ConfigFile *, ConfigEntry *, int, int *);
 int blacklist_config_run(ConfigFile *, ConfigEntry *, int);
+int blacklist_set_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs);
+int blacklist_set_config_run(ConfigFile *cf, ConfigEntry *ce, int type);
 void blacklist_free_conf(void);
 void delete_blacklist_block(Blacklist *e);
 void blacklist_md_free(ModData *md);
@@ -104,12 +117,13 @@ int blacklist_ip_change(Client *client, const char *oldip);
 int blacklist_quit(Client *client, MessageTag *mtags, const char *comment);
 int blacklist_preconnect(Client *client);
 void blacklist_resolver_callback(void *arg, int status, int timeouts, struct hostent *he);
-int blacklist_start_check(Client *client);
+int blacklist_start_check(Client *client, int recheck);
 int blacklist_dns_request(Client *client, Blacklist *bl);
 int blacklist_rehash(void);
 int blacklist_rehash_complete(void);
 void blacklist_set_handshake_delay(void);
 void blacklist_free_bluser_if_able(BLUser *bl);
+EVENT(blacklist_recheck);
 
 #define SetBLUser(x, y)	do { moddata_client(x, blacklist_md).ptr = y; } while(0)
 #define BLUSER(x)	((BLUser *)moddata_client(x, blacklist_md).ptr)
@@ -117,6 +131,7 @@ void blacklist_free_bluser_if_able(BLUser *bl);
 MOD_TEST()
 {
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, blacklist_config_test);
+	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, blacklist_set_config_test);
 
 	CallbackAdd(modinfo->handle, CALLBACKTYPE_BLACKLIST_CHECK, blacklist_start_check);
 	return MOD_SUCCESS;
@@ -146,7 +161,19 @@ MOD_INIT()
 		return MOD_FAILED;
 	}
 
+	memset(&mreq, 0, sizeof(mreq));
+	mreq.name = "blacklistrecheck";
+	mreq.type = MODDATATYPE_CLIENT;
+	blacklistrecheck_md = ModDataAdd(modinfo->handle, mreq);
+	if (!blacklistrecheck_md)
+	{
+		config_error("[blacklist] failed adding moddata for blacklistrecheck. "
+		             "Do you perhaps still have third/blacklistrecheck loaded? That module is no longer needed!");
+		return MOD_FAILED;
+	}
+
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, blacklist_config_run);
+	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN, 0, blacklist_set_config_run);
 	HookAdd(modinfo->handle, HOOKTYPE_HANDSHAKE, 0, blacklist_handshake);
 	HookAdd(modinfo->handle, HOOKTYPE_IP_CHANGE, 0, blacklist_ip_change);
 	HookAdd(modinfo->handle, HOOKTYPE_PRE_LOCAL_CONNECT, 0, blacklist_preconnect);
@@ -154,6 +181,8 @@ MOD_INIT()
 	HookAdd(modinfo->handle, HOOKTYPE_REHASH_COMPLETE, 0, blacklist_rehash_complete);
 	HookAdd(modinfo->handle, HOOKTYPE_LOCAL_QUIT, 0, blacklist_quit);
 
+	EventAdd(modinfo->handle, "blacklist_recheck", blacklist_recheck, NULL, 2000, 0);
+
 	return MOD_SUCCESS;
 }
 
@@ -237,6 +266,7 @@ void delete_blacklist_block(Blacklist *e)
 
 	safe_free(e->name);
 	safe_free(e->reason);
+	safe_free_all_ban_actions(e->action);
 	free_security_group(e->except);
 
 	safe_free(e);
@@ -247,7 +277,7 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 	ConfigEntry	*cep, *cepp, *ceppp;
 	int errors = 0;
 	char has_reason = 0, has_ban_time = 0, has_action = 0;
-	char has_dns_type = 0, has_dns_reply = 0, has_dns_name = 0;
+	char has_dns_type = 0, has_dns_reply = 0, has_dns_name = 0, has_recheck = 0;
 
 	if (type != CONFIG_MAIN)
 		return 0;
@@ -379,12 +409,7 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 				continue;
 			}
 			has_action = 1;
-			if (!banact_stringtoval(cep->value))
-			{
-				config_error("%s:%i: blacklist::action has unknown action type '%s'",
-					cep->file->filename, cep->line_number, cep->value);
-				errors++;
-			}
+			errors += test_ban_action_config(cep);
 		}
 		else if (!strcmp(cep->name, "ban-time"))
 		{
@@ -406,6 +431,16 @@ int blacklist_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 			}
 			has_reason = 1;
 		}
+		else if (!strcmp(cep->name, "recheck"))
+		{
+			if (has_recheck)
+			{
+				config_warn_duplicate(cep->file->filename,
+					cep->line_number, "blacklist::recheck");
+				continue;
+			}
+			has_recheck = 1;
+		}
 		else
 		{
 			config_error_unknown(cep->file->filename, cep->line_number,
@@ -467,9 +502,8 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	d = safe_alloc(sizeof(Blacklist));
 	safe_strdup(d->name, ce->value);
 	/* set some defaults */
-	d->action = BAN_ACT_KILL;
-	safe_strdup(d->reason, "Your IP is on a DNS Blacklist");
 	d->ban_time = 3600;
+	d->recheck = 1;
 	
 	/* assume dns for now ;) */
 	d->backend_type = BLACKLIST_BACKEND_DNS;
@@ -529,7 +563,7 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		}
 		else if (!strcmp(cep->name, "action"))
 		{
-			d->action = banact_stringtoval(cep->value);
+			parse_ban_action_config(cep, &d->action);
 		}
 		else if (!strcmp(cep->name, "ban-time"))
 		{
@@ -543,6 +577,10 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 		{
 			conf_match_block(cf, cep, &d->except);
 		}
+		else if (!strcmp(cep->name, "recheck"))
+		{
+			d->recheck = config_checkval(cep->value, CFG_YESNO);
+		}
 	}
 
 	AddListItem(d, conf_blacklist);
@@ -550,6 +588,101 @@ int blacklist_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 	return 0;
 }
 
+/** Test the set::blacklist configuration */
+int blacklist_set_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
+{
+	int errors = 0;
+	ConfigEntry *cep, *cepp;
+
+	if (type != CONFIG_SET)
+		return 0;
+	
+	/* We are only interrested in set::blacklist.. */
+	if (!ce || !ce->name || strcmp(ce->name, "blacklist"))
+		return 0;
+	
+	for (cep = ce->items; cep; cep = cep->next)
+	{
+		if (!strcmp(cep->name, "recheck-time-first"))
+		{
+			int v;
+			if (!cep->value)
+			{
+				config_error("%s:%i: set::blacklist::recheck-time-first with no value",
+					cep->file->filename, cep->line_number);
+				errors++;
+			}
+			if (!strcmp(cep->value, "never"))
+			{
+				config_error("%s:%i: if you want to disable blacklist rechecks, then you "
+				             "should set set::blacklist::recheck-time to 'never' and "
+				             "don't set set::blacklist::recheck-time-first.",
+				             cep->file->filename, cep->line_number);
+				errors++;
+				continue;
+			}
+			v = config_checkval(cep->value, CFG_TIME);
+			if (v < 60)
+			{
+				config_error("%s:%i: set::blacklist::recheck-time-first cannot be less than 60 seconds",
+					cep->file->filename, cep->line_number);
+				errors++;
+			}
+		} else
+		if (!strcmp(cep->name, "recheck-time"))
+		{
+			int v;
+			if (!cep->value)
+			{
+				config_error("%s:%i: set::blacklist::recheck-time with no value",
+					cep->file->filename, cep->line_number);
+				errors++;
+			}
+			if (strcmp(cep->value, "never"))
+			{
+				v = config_checkval(cep->value, CFG_TIME);
+				if (v < 60)
+				{
+					config_error("%s:%i: set::blacklist::recheck-time cannot be less than 60 seconds",
+						cep->file->filename, cep->line_number);
+					errors++;
+				}
+			}
+		} else
+		{
+			config_error("%s:%i: unknown directive set::blacklist::%s",
+				cep->file->filename, cep->line_number, cep->name);
+			errors++;
+			continue;
+		}
+	}
+	
+	*errs = errors;
+	return errors ? -1 : 1;
+}
+
+/* Configure ourselves based on the set::blacklist settings */
+int blacklist_set_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
+{
+	ConfigEntry *cep, *cepp;
+
+	if (type != CONFIG_SET)
+		return 0;
+	
+	/* We are only interrested in set::blacklist.. */
+	if (!ce || !ce->name || strcmp(ce->name, "blacklist"))
+		return 0;
+	
+	for (cep = ce->items; cep; cep = cep->next)
+	{
+		if (!strcmp(cep->name, "recheck-time"))
+			BLACKLIST_RECHECK_TIME = config_checkval(cep->value, CFG_TIME);
+		if (!strcmp(cep->name, "recheck-time-first"))
+			BLACKLIST_RECHECK_TIME_FIRST = config_checkval(cep->value, CFG_TIME);
+	}
+	return 1;
+}
+
 void blacklist_md_free(ModData *md)
 {
 	BLUser *bl = md->ptr;
@@ -562,17 +695,17 @@ void blacklist_md_free(ModData *md)
 
 int blacklist_handshake(Client *client)
 {
-	blacklist_start_check(client);
+	blacklist_start_check(client, 0);
 	return 0;
 }
 
 int blacklist_ip_change(Client *client, const char *oldip)
 {
-	blacklist_start_check(client);
+	blacklist_start_check(client, 0);
 	return 0;
 }
 
-int blacklist_start_check(Client *client)
+int blacklist_start_check(Client *client, int recheck)
 {
 	Blacklist *bl;
 
@@ -598,6 +731,9 @@ int blacklist_start_check(Client *client)
 		if (!BLUSER(client))
 			break;
 
+		if (recheck && !bl->recheck)
+			continue; /* blacklist::recheck is no */
+
 		/* Check if user is exempt (then don't bother checking) */
 		if (user_allowed_by_security_group(client, bl->except))
 			continue;
@@ -692,6 +828,7 @@ void blacklist_free_bluser_if_able(BLUser *bl)
 
 	safe_free(bl->save_opernotice);
 	safe_free(bl->save_reason);
+	free_all_ban_actions(bl->save_action);
 	safe_free(bl);
 }
 
@@ -747,7 +884,7 @@ int blacklist_parse_reply(struct hostent *he, int entry)
  * from blacklist_preconnect() for softbans that need to be delayed
  * as to give the user the opportunity to do SASL Authentication.
  */
-int blacklist_action(Client *client, char *opernotice, BanAction ban_action, char *ban_reason, long ban_time,
+int blacklist_action(Client *client, char *opernotice, BanAction *ban_action, char *ban_reason, long ban_time,
                      char *blacklist, char *blacklist_dns_name, int blacklist_dns_reply)
 {
 	unreal_log_raw(ULOG_INFO, "blacklist", "BLACKLIST_HIT", client,
@@ -755,12 +892,10 @@ int blacklist_action(Client *client, char *opernotice, BanAction ban_action, cha
 	               log_data_string("blacklist_name", blacklist),
 	               log_data_string("blacklist_dns_name", blacklist_dns_name),
 	               log_data_integer("blacklist_dns_reply", blacklist_dns_reply),
-	               log_data_string("ban_action", banact_valtostring(ban_action)),
+	               log_data_string("ban_action", ban_actions_to_string(ban_action)),
 	               log_data_string("ban_reason", ban_reason),
 	               log_data_integer("ban_time", ban_time));
-	if (ban_action == BAN_ACT_WARN)
-		return 0;
-	return place_host_ban(client, ban_action, ban_reason, ban_time);
+	return take_action(client, ban_action, ban_reason, ban_time, 0, NULL);
 }
 
 void blacklist_hit(Client *client, Blacklist *bl, int reply)
@@ -797,10 +932,10 @@ void blacklist_hit(Client *client, Blacklist *bl, int reply)
 
 	buildvarstring(bl->reason, banbuf, sizeof(banbuf), name, value);
 
-	if (IsSoftBanAction(bl->action) && blu)
+	if (only_soft_actions(bl->action) && blu)
 	{
 		/* For soft bans, delay the action until later (so user can do SASL auth) */
-		blu->save_action = bl->action;
+		blu->save_action = duplicate_ban_actions(bl->action);
 		blu->save_tkltime = bl->ban_time;
 		safe_strdup(blu->save_opernotice, opernotice);
 		safe_strdup(blu->save_reason, banbuf);
@@ -883,9 +1018,42 @@ int blacklist_preconnect(Client *client)
 		return HOOK_CONTINUE; /* yup, so the softban does not apply. */
 
 	if (blacklist_action(client, blu->save_opernotice, blu->save_action, blu->save_reason, blu->save_tkltime,
-	                     blu->save_blacklist, blu->save_blacklist_dns_name, blu->save_blacklist_dns_reply))
+	                     blu->save_blacklist, blu->save_blacklist_dns_name, blu->save_blacklist_dns_reply) > 0)
 	{
 		return HOOK_DENY;
 	}
 	return HOOK_CONTINUE; /* exempt */
 }
+
+void blacklist_recheck_user(Client *client)
+{
+	SetLastBLCheck(client, TStime());
+	blacklist_start_check(client, 1);
+}
+
+EVENT(blacklist_recheck)
+{
+	Client *client;
+	time_t last_check;
+
+	if (BLACKLIST_RECHECK_TIME == 0)
+		return;
+
+	list_for_each_entry(client, &lclient_list, lclient_node)
+	{
+		/* Only check connected users */
+		if (!IsUser(client))
+			continue;
+
+		last_check = LastBLCheck(client);
+		if ((last_check == 0) && (TStime() - client->local->creationtime >= BLACKLIST_RECHECK_TIME_FIRST))
+		{
+			/* First time: check after 60 seconds already */
+			blacklist_recheck_user(client);
+		} else /* After that, check every <...> seconds */
+		if (last_check && (TStime() - last_check) >= BLACKLIST_RECHECK_TIME)
+		{
+			blacklist_recheck_user(client);
+		}
+	}
+}
