@@ -340,7 +340,7 @@ void cmd_message(Client *client, MessageTag *recv_mtags, int parc, const char *p
 
 			new_message(client, recv_mtags, &mtags);
 
-			RunHook(HOOKTYPE_PRE_CHANMSG, client, channel, mtags, text, sendtype);
+			RunHook(HOOKTYPE_PRE_CHANMSG, client, channel, &mtags, text, sendtype);
 
 			if (!text)
 			{
@@ -616,14 +616,10 @@ int ban_version(Client *client, const char *text)
 
 	if ((ban = find_ban(NULL, ctcp_reply, CONF_BAN_VERSION)))
 	{
-		if (IsSoftBanAction(ban->action) && IsLoggedIn(client))
-			return 0; /* soft ban does not apply to us, we are logged in */
-
 		if (find_tkl_exception(TKL_BAN_VERSION, client))
 			return 0; /* we are exempt */
 
-		place_host_ban(client, ban->action, ban->reason, BAN_VERSION_TKL_TIME);
-		return 1;
+		return take_action(client, ban->action, ban->reason, BAN_VERSION_TKL_TIME, 0, NULL);
 	}
 
 	return 0;
