@@ -140,7 +140,7 @@ CMD_FUNC(cmd_nick_remote)
 	if (!IsULine(client) && (tklban = find_qline(client, nick, &ishold)) && !ishold)
 	{
 		unreal_log(ULOG_INFO, "nick", "QLINE_NICK_REMOTE", client,
-			   "Banned nick $nick [$ip] from server $server ($reason)",
+			   "Banned nick $nick used by $client.details, allowed through by server $server ($reason)",
 			   log_data_string("nick", parv[1]),
 			   log_data_string("ip", GetIP(client)),
 			   log_data_client("server", client->uplink),
@@ -287,7 +287,7 @@ CMD_FUNC(cmd_nick_local)
 			add_fake_lag(client, 4000); /* lag them up */
 			sendnumeric(client, ERR_ERRONEUSNICKNAME, nick, tklban->ptr.nameban->reason);
 			unreal_log(ULOG_INFO, "nick", "QLINE_NICK_LOCAL_ATTEMPT", client,
-				   "Attempt to use banned nick $nick [$ip] blocked ($reason)",
+				   "Attempt to use banned nick $nick by $client.details blocked ($reason)",
 				   log_data_string("nick", parv[1]),
 				   log_data_string("ip", GetIP(client)),
 				   log_data_client("server", client->uplink),
@@ -1051,8 +1051,9 @@ int _register_user(Client *client)
 	spamfilter_build_user_string(spamfilter_user, client->name, client);
 	if (match_spamfilter(client, spamfilter_user, SPAMF_USER, NULL, NULL, 0, &savetkl))
 	{
-		if (savetkl && ((savetkl->ptr.spamfilter->action == BAN_ACT_VIRUSCHAN) ||
-				(savetkl->ptr.spamfilter->action == BAN_ACT_SOFT_VIRUSCHAN)))
+		if (savetkl &&
+		    (has_actions_of_type(savetkl->ptr.spamfilter->action, BAN_ACT_VIRUSCHAN) ||
+		     has_actions_of_type(savetkl->ptr.spamfilter->action, BAN_ACT_SOFT_VIRUSCHAN)))
 		{
 			/* 'viruschan' action:
 			 * Continue with registering the client, and at the end
