@@ -113,7 +113,7 @@ int rpc_json_expand_client_server(Client *client, int detail, json_t *j, json_t
 const char *rrpc_md_serialize(ModData *m);
 void rrpc_md_unserialize(const char *str, ModData *m);
 void rrpc_md_free(ModData *m);
-int rpc_reconfigure_web_listener(ConfigItem_listen *listener);
+int rpc_config_listener(ConfigItem_listen *listener);
 
 /* Macros */
 #define RPC_PORT(client)  ((client->local && client->local->listener) ? client->local->listener->rpc_options : 0)
@@ -163,7 +163,7 @@ MOD_INIT()
 	HookAdd(modinfo->handle, HOOKTYPE_SERVER_QUIT, 0, rpc_handle_server_quit);
 	HookAdd(modinfo->handle, HOOKTYPE_FREE_CLIENT, 0, rpc_handle_free_client);
 	HookAdd(modinfo->handle, HOOKTYPE_JSON_EXPAND_CLIENT_SERVER, 0, rpc_json_expand_client_server);
-	HookAdd(modinfo->handle, HOOKTYPE_RECONFIGURE_WEB_LISTENER, 0, rpc_reconfigure_web_listener);
+	HookAdd(modinfo->handle, HOOKTYPE_CONFIG_LISTENER, 0, rpc_config_listener);
 
 	memset(&r, 0, sizeof(r));
 	r.method = "rpc.info";
@@ -325,13 +325,12 @@ int rpc_config_run_ex_listen(ConfigFile *cf, ConfigEntry *ce, int type, void *pt
 
 	l = (ConfigItem_listen *)ptr;
 	l->options |= LISTENER_NO_CHECK_CONNECT_FLOOD;
-	rpc_listener_set_handler(l);
 	l->rpc_options = 1;
 
 	return 1;
 }
 
-int rpc_reconfigure_web_listener(ConfigItem_listen *listener)
+int rpc_config_listener(ConfigItem_listen *listener)
 {
 	if (listener->rpc_options)
 		rpc_listener_set_handler(listener);
@@ -381,7 +380,7 @@ int rpc_config_test_rpc_user(ConfigFile *cf, ConfigEntry *ce, int type, int *err
 	}
 	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->name, "match"))
+		if (!strcmp(cep->name, "match") || !strcmp(cep->name, "mask"))
 		{
 			has_match = 1;
 			test_match_block(cf, cep, &errors);
@@ -389,8 +388,13 @@ int rpc_config_test_rpc_user(ConfigFile *cf, ConfigEntry *ce, int type, int *err
 		if (!strcmp(cep->name, "password"))
 		{
 			has_password = 1;
-			if (Auth_CheckError(cep) < 0)
+			if (Auth_CheckError(cep, 0) < 0)
 				errors++;
+		} else
+		{
+			config_error_unknown(cep->file->filename,
+				cep->line_number, "rpc-user", cep->name);
+			errors++;
 		}
 	}
 
@@ -413,7 +417,7 @@ int rpc_config_run_rpc_user(ConfigFile *cf, ConfigEntry *ce, int type)
 
 	for (cep = ce->items; cep; cep = cep->next)
 	{
-		if (!strcmp(cep->name, "match"))
+		if (!strcmp(cep->name, "match") || !strcmp(cep->name, "mask"))
 		{
 			conf_match_block(cf, cep, &e->match);
 		} else
