@@ -340,6 +340,8 @@ int write_tkldb(void)
 			{
 				if (tkl->flags & TKL_FLAG_CONFIG)
 					continue; /* config entry */
+				if (IsCentralSpamfilter(tkl))
+					continue;
 				if (!write_tkline(db, tmpfname, tkl)) // write_tkline() closes the db on errors itself
 					return 0;
 			}
@@ -430,7 +432,7 @@ int write_tkline(UnrealDB *db, const char *tmpfname, TKL *tkl)
 	{
 		char *match_type = unreal_match_method_valtostr(tkl->ptr.spamfilter->match->type);
 		char *target = spamfilter_target_inttostring(tkl->ptr.spamfilter->target);
-		char action = banact_valtochar(tkl->ptr.spamfilter->action);
+		char action = banact_valtochar(tkl->ptr.spamfilter->action->action); // NOTE: only writes single action value
 
 		W_SAFE(unrealdb_write_str(db, match_type));
 		W_SAFE(unrealdb_write_str(db, tkl->ptr.spamfilter->match->str));
@@ -699,8 +701,8 @@ int read_tkldb(void)
 
 			/* Action */
 			R_SAFE(unrealdb_read_char(db, &c));
-			tkl->ptr.spamfilter->action = banact_chartoval(c);
-			if (!tkl->ptr.spamfilter->action)
+			tkl->ptr.spamfilter->action = banact_value_to_struct(banact_chartoval(c)); // NOTE: only reads single action value
+			if (!tkl->ptr.spamfilter->action || banact_config_only(tkl->ptr.spamfilter->action->action))
 			{
 				config_warn("[tkldb] Spamfilter '%s' without valid action (%c) -- spamfilter entry not added",
 					tkl->ptr.spamfilter->match->str, c);
@@ -712,7 +714,7 @@ int read_tkldb(void)
 			tkl->ptr.spamfilter->tkl_duration = v;
 
 			if (find_tkl_spamfilter(tkl->type, tkl->ptr.spamfilter->match->str,
-			                        tkl->ptr.spamfilter->action,
+			                        tkl->ptr.spamfilter->action->action,
 			                        tkl->ptr.spamfilter->target))
 			{
 				do_not_add = 1;
@@ -720,17 +722,25 @@ int read_tkldb(void)
 
 			if (!do_not_add)
 			{
-				tkl_add_spamfilter(tkl->type, tkl->ptr.spamfilter->target,
+				tkl_add_spamfilter(tkl->type, NULL, tkl->ptr.spamfilter->target,
 				                   tkl->ptr.spamfilter->action,
 				                   tkl->ptr.spamfilter->match,
+				                   NULL,
+				                   NULL,
 				                   tkl->set_by, tkl->expire_at, tkl->set_at,
 				                   tkl->ptr.spamfilter->tkl_duration,
 				                   tkl->ptr.spamfilter->tkl_reason,
 				                   0);
-				/* tkl_add_spamfilter() does not copy the match but assign it.
-				 * so set to NULL here to avoid a read-after-free later on.
+				/* Further down in the code we free fields of the TKL entry,
+				 * this is generally fine since almost all fields are copied
+				 * by tkl_add_spamfilter(), however some are not.
+				 * For the following types tkl_add_spamfilter() actually
+				 * steals the reference, instead of duplicating,
+				 * so we have to set them to NULL here:
 				 */
 				tkl->ptr.spamfilter->match = NULL;
+				tkl->ptr.spamfilter->except = NULL;
+				tkl->ptr.spamfilter->action = NULL;
 			}
 		} else
 		{
