@@ -26,17 +26,6 @@ ModuleHeader MOD_HEADER
 
 #define WSU(client)	((WebSocketUser *)moddata_client(client, websocket_md).ptr)
 
-/* used to parse http Forwarded header (RFC 7239) */
-#define IPLEN 48
-#define FHEADER_NAMELEN	20
-
-struct HTTPForwardedHeader
-{
-	int secure;
-	char hostname[HOSTLEN+1];
-	char ip[IPLEN+1];
-};
-
 /* Forward declarations - public functions */
 int _websocket_handle_websocket(Client *client, WebRequest *web, const char *readbuf2, int length2, int callback(Client *client, char *buf, int len));
 int _websocket_create_packet(int opcode, char **buf, int *len);
@@ -47,6 +36,8 @@ int websocket_handle_packet(Client *client, const char *readbuf, int length, int
 int websocket_handle_packet_ping(Client *client, const char *buf, int len);
 int websocket_handle_packet_pong(Client *client, const char *buf, int len);
 int websocket_send_pong(Client *client, const char *buf, int len);
+const char *websocket_mdata_serialize(ModData *m);
+void websocket_mdata_unserialize(const char *str, ModData *m);
 void websocket_mdata_free(ModData *m);
 
 /* Global variables */
@@ -75,10 +66,10 @@ MOD_INIT()
 
 	memset(&mreq, 0, sizeof(mreq));
 	mreq.name = "websocket";
-	mreq.serialize = NULL;
-	mreq.unserialize = NULL;
+	mreq.serialize = websocket_mdata_serialize;
+	mreq.unserialize = websocket_mdata_unserialize;
 	mreq.free = websocket_mdata_free;
-	mreq.sync = 0;
+	mreq.sync = MODDATA_SYNC_EARLY;
 	mreq.type = MODDATATYPE_CLIENT;
 	websocket_md = ModDataAdd(modinfo->handle, mreq);
 
@@ -104,7 +95,7 @@ int _websocket_handle_websocket(Client *client, WebRequest *web, const char *rea
 	char *ptr;
 	int length;
 	int length1 = WSU(client)->lefttoparselen;
-	char readbuf[4096];
+	char readbuf[MAXLINELENGTH];
 
 	length = length1 + length2;
 	if (length > sizeof(readbuf)-1)
@@ -159,6 +150,7 @@ int websocket_handle_packet(Client *client, const char *readbuf, int length, int
 	int total_packet_size;
 	char *payload = NULL;
 	static char payloadbuf[READBUF_SIZE];
+	int maskkeylen = 4;
 
 	if (length < 4)
 	{
@@ -174,19 +166,27 @@ int websocket_handle_packet(Client *client, const char *readbuf, int length, int
 
 	/* actually 'fin' is unused.. we don't care. */
 
-	if (!masked)
+	/* Masked. According to RFC6455 page 29:
+	 * "All frames sent from client to server have this bit set to 1."
+	 * But in practice i see that for PONG this may not always be
+	 * true, so let's make an exception for that...
+	 */
+	if (!masked && (opcode != WSOP_PONG))
 	{
 		dead_socket(client, "WebSocket packet not masked");
 		return -1; /* Having the masked bit set is required (RFC6455 p29) */
 	}
 
+	if (!masked)
+		maskkeylen = 0;
+
 	if (len == 127)
 	{
 		dead_socket(client, "WebSocket packet with insane size");
 		return -1; /* Packets requiring 64bit lengths are not supported. Would be insane. */
 	}
 
-	total_packet_size = len + 2 + 4; /* 2 for header, 4 for mask key, rest for payload */
+	total_packet_size = len + 2 + maskkeylen; /* 2 for header, 4 for mask key, rest for payload */
 
 	/* Early (minimal) length check */
 	if (length < total_packet_size)
@@ -208,17 +208,20 @@ int websocket_handle_packet(Client *client, const char *readbuf, int length, int
 		p += 2; /* advance pointer 2 bytes */
 
 		/* Need to check the length again, now it has changed: */
-		if (length < len + 4 + 4)
+		if (length < len + 4 + maskkeylen)
 		{
 			/* WebSocket frame too short */
 			return 0;
 		}
 		/* And update the packet size */
-		total_packet_size = len + 4 + 4; /* 4 for header, 4 for mask key, rest for payload */
+		total_packet_size = len + 4 + maskkeylen; /* 4 for header, 4 for mask key, rest for payload */
 	}
 
-	memcpy(maskkey, p, 4);
-	p+= 4;
+	if (masked)
+	{
+		memcpy(maskkey, p, maskkeylen);
+		p+= maskkeylen;
+	}
 
 	if (len > 0)
 	{
@@ -226,7 +229,7 @@ int websocket_handle_packet(Client *client, const char *readbuf, int length, int
 		payload = payloadbuf;
 	} /* else payload is NULL */
 
-	if (len > 0)
+	if (masked && (len > 0))
 	{
 		/* Unmask this thing (page 33, section 5.3) */
 		int n;
@@ -482,3 +485,28 @@ void websocket_mdata_free(ModData *m)
 		safe_free(m->ptr);
 	}
 }
+
+/** This only serializes wsu->type atm */
+const char *websocket_mdata_serialize(ModData *m)
+{
+	static char buf[32];
+	WebSocketUser *wsu = m->ptr;
+
+	if (!wsu)
+		return NULL; /* not set */
+
+	snprintf(buf, sizeof(buf), "%d", wsu->type);
+	return buf;
+}
+
+/** This only sets wsu->type atm */
+void websocket_mdata_unserialize(const char *str, ModData *m)
+{
+	WebSocketUser *wsu;
+	if (m->ptr)
+		websocket_mdata_free(m);
+	if (BadPtr(str))
+		return; /* empty/freed */
+	m->ptr = wsu = safe_alloc(sizeof(WebSocketUser));
+	wsu->type = atoi(str);
+}
