@@ -33,18 +33,21 @@ void unreal_delete_masks(ConfigItem_mask *m)
 	}
 }
 
+void unreal_add_mask_string(ConfigItem_mask **head, const char *name)
+{
+	ConfigItem_mask *m = safe_alloc(sizeof(ConfigItem_mask));
+	safe_strdup(m->mask, name);
+	add_ListItem((ListStruct *)m, (ListStruct **)head);
+}
+
 /** Internal function to add one individual mask to the list */
 static void unreal_add_mask(ConfigItem_mask **head, ConfigEntry *ce)
 {
-	ConfigItem_mask *m = safe_alloc(sizeof(ConfigItem_mask));
-
 	/* Since we allow both mask "xyz"; and mask { abc; def; };... */
 	if (ce->value)
-		safe_strdup(m->mask, ce->value);
+		unreal_add_mask_string(head, ce->value);
 	else
-		safe_strdup(m->mask, ce->name);
-
-	add_ListItem((ListStruct *)m, (ListStruct **)head);
+		unreal_add_mask_string(head, ce->name);
 }
 
 /** Add mask entries from config */
@@ -173,7 +176,7 @@ int unreal_mask_match_string(const char *name, ConfigItem_mask *mask)
 	return retval;
 }
 
-#define CheckNullX(x) if ((!(x)->value) || (!(*((x)->value)))) { config_error("%s:%i: missing parameter", (x)->file->filename, (x)->line_number); *errors = *errors + 1; return 0; }
+#define CheckNullX(x) if ((!(x)->value) || (!(*((x)->value)))) { config_error("%s:%i: missing parameter", (x)->file->filename, (x)->line_number); (*errors)++; return 0; }
 int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 {
 	ConfigEntry *cepp;
@@ -206,7 +209,7 @@ int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 		{
 			config_error("%s:%i: %s needs to be a value of 1-10000",
 				cep->file->filename, cep->line_number, cep->name);
-			*errors = *errors + 1;
+			(*errors)++;
 		}
 	} else
 	if (!strcmp(cep->name, "connect-time") || !strcmp(cep->name, "exclude-connect-time"))
@@ -221,7 +224,7 @@ int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 		{
 			config_error("%s:%i: %s needs to be a time value (and more than 0 seconds)",
 				cep->file->filename, cep->line_number, cep->name);
-			*errors = *errors + 1;
+			(*errors)++;
 		}
 	} else
 	if (!strcmp(cep->name, "mask") || !strcmp(cep->name, "include-mask") || !strcmp(cep->name, "exclude-mask"))
@@ -236,7 +239,7 @@ int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 				             "Perhaps you meant to use it in security-group { %s ... } directly?",
 				             cepp->file->filename, cepp->line_number,
 				             cepp->name);
-				*errors = *errors + 1;
+				(*errors)++;
 			}
 		}
 	} else
@@ -246,6 +249,18 @@ int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 	if (!strcmp(cep->name, "security-group") || !strcmp(cep->name, "exclude-security-group"))
 	{
 	} else
+	if (!strcmp(cep->name, "rule") || !strcmp(cep->name, "exclude-rule"))
+	{
+		int val = crule_test(cep->value);
+		if (val)
+		{
+			config_error("%s:%i: rule contains an invalid expression: %s",
+				cep->file->filename,
+				cep->line_number,
+				crule_errstring(val));
+			(*errors)++;
+		}
+	} else
 	{
 		/* Let's see if an extended server ban exists for this item... */
 		Extban *extban;
@@ -415,6 +430,11 @@ int conf_match_item(ConfigFile *conf, ConfigEntry *cep, SecurityGroup **block)
 	{
 		unreal_add_names(&s->security_group, cep);
 	}
+	else if (!strcmp(cep->name, "rule"))
+	{
+		safe_strdup(s->prettyrule, cep->value);
+		s->rule = crule_parse(s->prettyrule);
+	}
 	else if (!strcmp(cep->name, "exclude-webirc"))
 		s->exclude_webirc = config_checkval(cep->value, CFG_YESNO);
 	else if (!strcmp(cep->name, "exclude-websocket"))
@@ -438,6 +458,11 @@ int conf_match_item(ConfigFile *conf, ConfigEntry *cep, SecurityGroup **block)
 	{
 		unreal_add_names(&s->security_group, cep);
 	}
+	else if (!strcmp(cep->name, "exclude-rule"))
+	{
+		safe_strdup(s->exclude_prettyrule, cep->value);
+		s->exclude_rule = crule_parse(s->exclude_prettyrule);
+	}
 	else
 	{
 		/* Let's see if an extended server ban exists for this item... this needs to be LAST! */
@@ -596,6 +621,10 @@ void free_security_group(SecurityGroup *s)
 	unreal_delete_masks(s->exclude_mask);
 	free_entire_name_list(s->security_group);
 	free_entire_name_list(s->exclude_security_group);
+	safe_crule_free(s->rule);
+	safe_crule_free(s->exclude_rule);
+	safe_free(s->prettyrule);
+	safe_free(s->exclude_prettyrule);
 	free_entire_name_list(s->ip);
 	free_entire_name_list(s->exclude_ip);
 	free_nvplist(s->extended);
@@ -618,6 +647,16 @@ SecurityGroup *duplicate_security_group(SecurityGroup *s)
 	n->exclude_mask = unreal_duplicate_masks(s->exclude_mask);
 	n->security_group = duplicate_name_list(s->security_group);
 	n->exclude_security_group = duplicate_name_list(s->exclude_security_group);
+	if (s->prettyrule)
+	{
+		safe_strdup(n->prettyrule, s->prettyrule);
+		n->rule = crule_parse(n->prettyrule);
+	}
+	if (s->exclude_prettyrule)
+	{
+		safe_strdup(n->exclude_prettyrule, s->exclude_prettyrule);
+		n->exclude_rule = crule_parse(n->exclude_prettyrule);
+	}
 	n->ip = duplicate_name_list(s->exclude_ip);
 	n->extended = duplicate_nvplist(s->extended);
 	n->exclude_extended = duplicate_nvplist(s->exclude_extended);
@@ -705,7 +744,7 @@ int test_extended_list(Extban *extban, ConfigEntry *cep, int *errors)
 		{
 			config_error("%s:%i: %s has an invalid value",
 			             cep->file->filename, cep->line_number, cep->name);
-			*errors = *errors + 1;
+			(*errors)++;
 			return 0;
 		}
 	}
@@ -720,7 +759,7 @@ int test_extended_list(Extban *extban, ConfigEntry *cep, int *errors)
 		{
 			config_error("%s:%i: %s has an invalid value",
 			             cep->file->filename, cep->line_number, cep->name);
-			*errors = *errors + 1;
+			(*errors)++;
 			return 0;
 		}
 	}
@@ -743,6 +782,17 @@ int user_allowed_by_security_group_list(Client *client, NameList *l)
 	return 0;
 }
 
+/** Helper for security-group::rule and mask::rule */
+int user_allowed_by_rule(Client *client, CRuleNode *rule)
+{
+	crule_context context;
+
+	memset(&context, 0, sizeof(context));
+	context.client = client;
+
+	return crule_eval(&context, rule);
+}
+
 /** Returns 1 if the user is OK as far as the security-group is concerned.
  * @param client	The client to check
  * @param s		The security-group to check against
@@ -796,10 +846,12 @@ int user_allowed_by_security_group(Client *client, SecurityGroup *s)
 		goto user_not_allowed;
 	if (s->exclude_ip && unreal_match_iplist(client, s->exclude_ip))
 		goto user_not_allowed;
-	if (s->exclude_security_group && user_allowed_by_security_group_list(client, s->exclude_security_group))
+	if (s->exclude_rule && user_allowed_by_rule(client, s->exclude_rule))
 		goto user_not_allowed;
 	if (s->exclude_extended && user_matches_extended_list(client, s->exclude_extended))
 		goto user_not_allowed;
+	if (s->exclude_security_group && user_allowed_by_security_group_list(client, s->exclude_security_group))
+		goto user_not_allowed;
 
 	/* Then process INCLUSION criteria... */
 	if (s->identified && IsLoggedIn(client))
@@ -826,10 +878,12 @@ int user_allowed_by_security_group(Client *client, SecurityGroup *s)
 		goto user_allowed;
 	if (s->ip && unreal_match_iplist(client, s->ip))
 		goto user_allowed;
-	if (s->security_group && user_allowed_by_security_group_list(client, s->security_group))
+	if (s->rule && user_allowed_by_rule(client, s->rule))
 		goto user_allowed;
 	if (s->extended && user_matches_extended_list(client, s->extended))
 		goto user_allowed;
+	if (s->security_group && user_allowed_by_security_group_list(client, s->security_group))
+		goto user_allowed;
 
 user_not_allowed:
 	recursion_security_group--;
