@@ -264,7 +264,7 @@ void send_proto(Client *client, ConfigItem_link *aconf)
 		me.id, (long long)TStime());
 
 	/* Third line */
-	sendto_one(client, NULL, "PROTOCTL NICKCHARS=%s CHANNELCHARS=%s",
+	sendto_one(client, NULL, "PROTOCTL NICKCHARS=%s CHANNELCHARS=%s BIGLINES",
 		charsys_get_current_languages(),
 		allowed_channelchars_valtostr(iConf.allowed_channelchars));
 }
@@ -279,8 +279,8 @@ int remotecmdfilter(Client *client, int parc, const char *parv[])
 	/* no remote requests permitted from non-ircops */
 	if (MyUser(client) && !ValidatePermissionsForPath("server:remote",client,NULL,NULL,NULL) && !BadPtr(parv[1]))
 	{
-		parv[1] = NULL;
-		parc = 1;
+		sendnumeric(client, ERR_NOPRIVILEGES);
+		return 1; /* STOP */
 	}
 
 	/* same as above, but in case an old server forwards a request to us: we ignore it */
@@ -569,7 +569,6 @@ CMD_FUNC(cmd_rehash)
 				sendnotice(client, "A rehash is already in progress");
 				return;
 			}
-			unreal_log(ULOG_INFO, "config", "CONFIG_RELOAD", client, "Rehashing server configuration file [by: $client.details]");
 			remote_rehash_client = client;
 			/* fallthrough... so we deal with this the same way as local rehashes */
 		}
