@@ -33,9 +33,16 @@ struct Download
 	Download *prev, *next;
 	vFP callback;
 	void *callback_data;
+	char *url; /**< must be free()d by url_do_transfers_async() */
+	HttpMethod http_method;
+	char *body;
+	NameValuePrioList *request_headers;
+	int store_in_file;
 	FILE *file_fd;		/**< File open for writing (otherwise NULL) */
-	char filename[PATH_MAX];
-	char *url; /*< must be free()d by url_do_transfers_async() */
+	char *filename;
+	char *memory_data; /**< Memory for writing response (otherwise NULL) */
+	int memory_data_len; /**< Size of memory_data */
+	int memory_data_allocated; /**< Total allocated memory for 'memory_data' */
 	char errorbuf[512];
 	time_t cachetime;
 	char *hostname;		/**< Parsed hostname (from 'url') */
@@ -75,16 +82,16 @@ SSL_CTX *https_new_ctx(void);
 void unreal_https_connect_handshake(int fd, int revents, void *data);
 int https_connect(Download *handle);
 int https_fatal_tls_error(int ssl_error, int my_errno, Download *handle);
-void https_connect_send_header(Download *handle);
+int https_connect_send_header(Download *handle);
 void https_receive_response(int fd, int revents, void *data);
 int https_handle_response_header(Download *handle, char *readbuf, int n);
-int https_handle_response_file(Download *handle, char *readbuf, int n);
+int https_handle_response_body(Download *handle, char *readbuf, int n);
 void https_done(Download *handle);
 void https_done_cached(Download *handle);
 void https_redirect(Download *handle);
 int https_parse_header(char *buffer, int len, char **key, char **value, char **lastloc, int *end_of_request);
 char *url_find_end_of_request(char *header, int totalsize, int *remaining_bytes);
-void https_cancel(Download *handle, FORMAT_STRING(const char *pattern), ...)  __attribute__((format(printf,2,3)));
+int https_cancel(Download *handle, FORMAT_STRING(const char *pattern), ...)  __attribute__((format(printf,2,3)));
 
 void url_free_handle(Download *handle)
 {
@@ -96,7 +103,10 @@ void url_free_handle(Download *handle)
 	}
 	if (handle->file_fd)
 		fclose(handle->file_fd);
-	safe_free(handle->url);
+	safe_free(handle->filename);
+	safe_free(handle->memory_data);
+	safe_free(handle->body);
+	safe_free_nvplist(handle->request_headers);
 	safe_free(handle->hostname);
 	safe_free(handle->username);
 	safe_free(handle->password);
@@ -107,6 +117,7 @@ void url_free_handle(Download *handle)
 	safe_free(handle->lefttoparse);
 	safe_free(handle->redirect_new_location);
 	safe_free(handle->redirect_original_url);
+	safe_free(handle->url);
 	safe_free(handle);
 }
 
@@ -125,18 +136,27 @@ void url_cancel_handle_by_callback_data(void *ptr)
 	}
 }
 
-void https_cancel(Download *handle, FORMAT_STRING(const char *pattern), ...)
+/** Cancel and free the HTTPS request.
+ * @returns Always returns -1
+ */
+int https_cancel(Download *handle, FORMAT_STRING(const char *pattern), ...)
 {
 	va_list vl;
 	va_start(vl, pattern);
 	vsnprintf(handle->errorbuf, sizeof(handle->errorbuf), pattern, vl);
 	va_end(vl);
 	if (handle->callback)
-		handle->callback(handle->url, NULL, handle->errorbuf, 0, handle->callback_data);
+		handle->callback(handle->url, NULL, NULL, 0, handle->errorbuf, 0, handle->callback_data);
 	url_free_handle(handle);
+	return -1;
 }
 
 void download_file_async(const char *url, time_t cachetime, vFP callback, void *callback_data, char *original_url, int maxredirects)
+{
+	url_start_async(url, HTTP_METHOD_GET, NULL, NULL, 1, cachetime, callback, callback_data, original_url, maxredirects);
+}
+
+void url_start_async(const char *url, HttpMethod http_method, const char *body, NameValuePrioList *request_headers, int store_in_file, time_t cachetime, vFP callback, void *callback_data, char *original_url, int maxredirects)
 {
 	char *file;
 	const char *filename;
@@ -157,6 +177,9 @@ void download_file_async(const char *url, time_t cachetime, vFP callback, void *
 	safe_strdup(handle->url, url);
 	safe_strdup(handle->redirect_original_url, original_url);
 	handle->redirects_remaining = maxredirects;
+	handle->http_method = http_method;
+	safe_strdup(handle->body, body);
+	handle->store_in_file = store_in_file;
 	AddListItem(handle, downloads);
 
 	if (strncmp(url, "https://", 8))
@@ -176,21 +199,29 @@ void download_file_async(const char *url, time_t cachetime, vFP callback, void *
 	safe_strdup(handle->password, password);
 	safe_strdup(handle->document, document);
 
-	file = url_getfilename(url);
-	filename = unreal_getfilename(file);
-	tmp = unreal_mktemp(TMPDIR, filename ? filename : "download.conf");
-
-	handle->file_fd = fopen(tmp, "wb");
-	if (!handle->file_fd)
+	if (store_in_file)
 	{
-		https_cancel(handle, "Cannot create '%s': %s", tmp, strerror(ERRNO));
+		file = url_getfilename(url);
+		filename = unreal_getfilename(file);
+		tmp = unreal_mktemp(TMPDIR, filename ? filename : "download.conf");
+
+		handle->file_fd = fopen(tmp, "wb");
+		if (!handle->file_fd)
+		{
+			https_cancel(handle, "Cannot create '%s': %s", tmp, strerror(ERRNO));
+			safe_free(file);
+			return;
+		}
+
+		safe_strdup(handle->filename, tmp);
 		safe_free(file);
-		return;
+	} else {
+		handle->memory_data_allocated = URL_MEMORY_BACKED_CHUNK_SIZE;
+		handle->memory_data = safe_alloc(URL_MEMORY_BACKED_CHUNK_SIZE);
 	}
 
-	strlcpy(handle->filename, tmp, sizeof(handle->filename));
-	safe_free(file);
-
+	if (request_headers)
+		handle->request_headers = duplicate_nvplist(request_headers);
 
 	// todo: allocate handle, select en weetikt allemaal
 	// add to some global struct linkedlist, for timeouts
@@ -284,11 +315,7 @@ void unreal_https_connect_handshake(int fd, int revents, void *data)
 	SSL_set_tlsext_host_name(handle->ssl, handle->hostname);
 
 	if (https_connect(handle) < 0)
-	{
-		/* Some fatal error already */
-		https_cancel(handle, "TLS_connect() failed early");
-		return;
-	}
+		return; /* fatal error, handle is freed */
 
 	/* Is now connecting... */
 }
@@ -342,7 +369,12 @@ void https_connect_retry(int fd, int revents, void *data)
 	https_connect(handle);
 }
 
-// Based on unreal_tls_connect()
+/* Actually do the SSL_connect()
+ * Based on unreal_tls_connect() but different return values.
+ * @retval 1 connected
+ * @retval 0 in progress
+ * @retval -1 error, handle freed!
+ */
 int https_connect(Download *handle)
 {
 	int ssl_err;
@@ -372,21 +404,18 @@ int https_connect(Download *handle)
 				fd_setselect(handle->fd, FD_SELECT_WRITE, https_connect_retry, handle);
 				return 0;
 			default:
-				return https_fatal_tls_error(ssl_err, ERRNO, handle);
+				return https_fatal_tls_error(ssl_err, ERRNO, handle); /* -1 */
 		}
 		/* NOTREACHED */
-		return -1;
+		return 0;
 	}
 
 	/* We are connected now. */
 
 	if (!verify_certificate(handle->ssl, handle->hostname, &errstr))
-	{
-		https_cancel(handle, "TLS Certificate error for server: %s", errstr);
-		return -1;
-	}
-	https_connect_send_header(handle);
-	return 1;
+		return https_cancel(handle, "TLS Certificate error for server: %s", errstr); /* -1 */
+
+	return https_connect_send_header(handle);
 }
 
 /**
@@ -396,6 +425,7 @@ int https_connect(Download *handle)
  * @param where The location, one of the SAFE_SSL_* defines.
  * @param my_errno A preserved value of errno to pass to ssl_error_str().
  * @param client The client the error is associated with.
+ * @returns Always -1
  */
 int https_fatal_tls_error(int ssl_error, int my_errno, Download *handle)
 {
@@ -481,9 +511,9 @@ int url_parse(const char *url, char **hostname, int *port, char **username, char
 	return 1;
 }
 
-void https_connect_send_header(Download *handle)
+int https_connect_send_header(Download *handle)
 {
-	char buf[1024];
+	char buf[8192];
 	char hostandport[512];
 	int ssl_err;
 	char *host;
@@ -494,13 +524,46 @@ void https_connect_send_header(Download *handle)
 	snprintf(hostandport, sizeof(hostandport), "%s:%d", handle->hostname, handle->port);
 
 	/* Prepare the header */
-	snprintf(buf, sizeof(buf), "GET %s HTTP/1.1\r\n"
-	                    "User-Agent: UnrealIRCd %s\r\n"
-	                    "Host: %s\r\n"
-	                    "Connection: close\r\n",
-	                    handle->document,
-	                    VERSIONONLY,
-	                    hostandport);
+	if (handle->http_method == HTTP_METHOD_GET)
+	{
+		snprintf(buf, sizeof(buf), "GET %s HTTP/1.1\r\n"
+				    "User-Agent: UnrealIRCd %s\r\n"
+				    "Host: %s\r\n"
+				    "Connection: close\r\n",
+				    handle->document,
+				    VERSIONONLY,
+				    hostandport);
+	} else
+	if (handle->http_method == HTTP_METHOD_POST)
+	{
+		if (!handle->body || !strlen(handle->body))
+		{
+			snprintf(buf, sizeof(buf), "POST %s HTTP/1.1\r\n"
+					    "User-Agent: UnrealIRCd %s\r\n"
+					    "Host: %s\r\n"
+					    "Connection: close\r\n",
+					    handle->document,
+					    VERSIONONLY,
+					    hostandport);
+		} else {
+			char add_default_content_type = 0;
+			if (!find_nvplist(handle->request_headers, "Content-Type"))
+				add_default_content_type = 1;
+
+			snprintf(buf, sizeof(buf), "POST %s HTTP/1.1\r\n"
+					    "User-Agent: UnrealIRCd %s\r\n"
+					    "Host: %s\r\n"
+					    "%s"
+					    "Content-Length: %ld\r\n"
+					    "Connection: close\r\n",
+					    handle->document,
+					    VERSIONONLY,
+					    hostandport,
+					    add_default_content_type ? "Content-Type: application/x-www-form-urlencoded\r\n" : "",
+					    (long)strlen(handle->body));
+		}
+	} else
+		abort();
 	if (handle->username && handle->password)
 	{
 		char wbuf[128];
@@ -524,16 +587,32 @@ void https_connect_send_header(Download *handle)
 				 "If-Modified-Since: %s\r\n", datestr);
 		}
 	}
+	if (handle->request_headers)
+	{
+		NameValuePrioList *n;
+		char nbuf[256];
+
+		for (n = handle->request_headers; n; n = n->next)
+		{
+			if (n->value)
+				snprintf(nbuf, sizeof(nbuf), "%s: %s\r\n", n->name, n->value);
+			else
+				snprintf(nbuf, sizeof(nbuf), "%s:\r\n", n->name);
+			if (strlen(buf)+strlen(nbuf) > sizeof(buf)-8)
+				break;
+			strlcat(buf, nbuf, sizeof(buf));
+		}
+	}
 	strlcat(buf, "\r\n", sizeof(buf));
+	if (handle->body)
+		strlcat(buf, handle->body, sizeof(buf));
 
 	ssl_err = SSL_write(handle->ssl, buf, strlen(buf));
 	if (ssl_err < 0)
-	{
-		https_fatal_tls_error(ssl_err, ERRNO, handle);
-		return;
-	}
+		return https_fatal_tls_error(ssl_err, ERRNO, handle);
 	fd_setselect(handle->fd, FD_SELECT_WRITE, NULL, handle);
 	fd_setselect(handle->fd, FD_SELECT_READ, https_receive_response, handle);
+	return 1;
 }
 
 void https_receive_response(int fd, int revents, void *data)
@@ -580,7 +659,7 @@ void https_receive_response(int fd, int revents, void *data)
 	} else
 	if (handle->got_response)
 	{
-		if (!https_handle_response_file(handle, readbuf, n))
+		if (!https_handle_response_body(handle, readbuf, n))
 			return; /* handle is already freed! */
 	}
 }
@@ -701,7 +780,7 @@ int https_handle_response_header(Download *handle, char *readbuf, int n)
 		nextframe = url_find_end_of_request(netbuf2, totalsize, &remaining_bytes);
 		if (nextframe)
 		{
-			if (!https_handle_response_file(handle, nextframe, remaining_bytes))
+			if (!https_handle_response_body(handle, nextframe, remaining_bytes))
 				return 0;
 		}
 	}
@@ -715,7 +794,39 @@ int https_handle_response_header(Download *handle, char *readbuf, int n)
 	return 1;
 }
 
-int https_handle_response_file(Download *handle, char *readbuf, int pktsize)
+int https_handle_response_body_memory(Download *handle, const char *ptr, int write_sz)
+{
+	// DUPLICATE CODE: same as src/url_curl.c, well... sortof
+	int size_required = handle->memory_data_len + write_sz;
+
+	if (handle->memory_data == NULL)
+		return 0; /* Normally does not happen as it is preallocated, but could happen upon unwinding cancels.. */
+
+	if (size_required >= handle->memory_data_allocated - 1) // the -1 is for zero termination, even though it is binary..
+	{
+		int newsize = ((size_required / URL_MEMORY_BACKED_CHUNK_SIZE)+1)*URL_MEMORY_BACKED_CHUNK_SIZE;
+		char *newptr = realloc(handle->memory_data, newsize);
+		if (!newptr)
+		{
+			unreal_log(ULOG_ERROR, "url", "URL_DOWNLOAD_MEMORY", NULL, "Async URL callback failed when reading returned data: out of memory?");
+			safe_free(handle->memory_data);
+			handle->memory_data_len = 0;
+			handle->memory_data_allocated = 0;
+			return 0;
+		}
+		handle->memory_data = newptr;
+		handle->memory_data_allocated = newsize;
+		/* fill rest with zeroes, yeah.. no trust! ;D */
+		memset(handle->memory_data + handle->memory_data_len, 0, handle->memory_data_allocated - handle->memory_data_len);
+	}
+
+	memcpy(handle->memory_data + handle->memory_data_len, ptr, write_sz);
+	handle->memory_data_len += write_sz;
+	handle->memory_data[handle->memory_data_len] = '\0';
+	return write_sz;
+}
+
+int https_handle_response_body(Download *handle, char *readbuf, int pktsize)
 {
 	char *buf;
 	long long n;
@@ -727,7 +838,10 @@ int https_handle_response_file(Download *handle, char *readbuf, int pktsize)
 	if (handle->transfer_encoding == TRANSFER_ENCODING_NONE)
 	{
 		/* Ohh.. so easy! */
-		fwrite(readbuf, 1, pktsize, handle->file_fd);
+		if (handle->store_in_file == 0)
+			https_handle_response_body_memory(handle, readbuf, pktsize);
+		else if (handle->file_fd)
+			fwrite(readbuf, 1, pktsize, handle->file_fd);
 		return 1;
 	}
 
@@ -754,7 +868,10 @@ int https_handle_response_file(Download *handle, char *readbuf, int pktsize)
 		{
 			/* Eat it */
 			int eat = MIN(handle->chunk_remaining, n);
-			fwrite(buf, 1, eat, handle->file_fd);
+			if (handle->store_in_file == 0)
+				https_handle_response_body_memory(handle, buf, eat);
+			else if (handle->file_fd)
+				fwrite(buf, 1, eat, handle->file_fd);
 			n -= eat;
 			buf += eat;
 			handle->chunk_remaining -= eat;
@@ -833,18 +950,21 @@ void https_done(Download *handle)
 {
 	char *url = handle->redirect_original_url ? handle->redirect_original_url : handle->url;
 
-	fclose(handle->file_fd);
-	handle->file_fd = NULL;
+	if (handle->file_fd)
+	{
+		fclose(handle->file_fd);
+		handle->file_fd = NULL;
+	}
 
 	if (!handle->callback)
 		; /* No special action, request was cancelled */
 	else if (!handle->got_response)
-		handle->callback(url, NULL, "HTTPS response not received", 0, handle->callback_data);
+		handle->callback(url, NULL, NULL, 0, "HTTPS response not received", 0, handle->callback_data);
 	else
 	{
-		if (handle->last_modified > 0)
+		if ((handle->last_modified > 0) && handle->filename)
 			unreal_setfilemodtime(handle->filename, handle->last_modified);
-		handle->callback(url, handle->filename, NULL, 0, handle->callback_data);
+		handle->callback(url, handle->filename, handle->memory_data, handle->memory_data_len, NULL, 0, handle->callback_data);
 	}
 	url_free_handle(handle);
 	return;
@@ -854,10 +974,13 @@ void https_done_cached(Download *handle)
 {
 	char *url = handle->redirect_original_url ? handle->redirect_original_url : handle->url;
 
-	fclose(handle->file_fd);
-	handle->file_fd = NULL;
+	if (handle->file_fd)
+	{
+		fclose(handle->file_fd);
+		handle->file_fd = NULL;
+	}
 	if (handle->callback)
-		handle->callback(url, NULL, NULL, 1, handle->callback_data);
+		handle->callback(url, NULL, NULL, 0, NULL, 1, handle->callback_data);
 	url_free_handle(handle);
 }
 
@@ -873,8 +996,10 @@ void https_redirect(Download *handle)
 	if (handle->callback)
 	{
 		/* If still an outstanding request (not cancelled), follow the redirect.. */
-		download_file_async(handle->redirect_new_location, handle->cachetime, handle->callback, handle->callback_data,
-				    handle->url, handle->redirects_remaining);
+		url_start_async(handle->redirect_new_location, handle->http_method, handle->body,
+		                handle->request_headers, handle->store_in_file,
+		                handle->cachetime, handle->callback, handle->callback_data,
+				handle->url, handle->redirects_remaining);
 	}
 	/* Don't call the hook, just free this, the new redirect from above will call the hook later */
 	url_free_handle(handle);
@@ -1084,3 +1209,55 @@ void url_init(void)
 	}
 	EventAdd(NULL, "url_socket_timeout", url_socket_timeout, NULL, 500, 0);
 }
+
+char *urlencode(const char *s, char *wbuf, int wlen)
+{
+	const char hexchars[16] = "0123456789abcdef";
+	char *o = wbuf;
+
+	if (s == NULL)
+	{
+		if (wlen > 0)
+			*o = '\0';
+		return NULL;
+	}
+
+	for (; *s; s++)
+	{
+		if (((*s >= 'a') && (*s <= 'z')) ||
+		    ((*s >= 'A') && (*s <= 'Z')) ||
+		    ((*s >= '0') && (*s <= '9')) ||
+		    strchr("-._~", *s))
+		{
+			if (wlen-- <= 1)
+				break;
+			*o++ = *s;
+		} else
+		{
+			if (wlen <= 3)
+				break;
+			wlen -= 3;
+			*o++ = '%';
+			*o++ = hexchars[(*s >> 4) & 0xF];
+			*o++ = hexchars[*s & 0xF];
+		}
+	}
+	/* And zero-terminate.. */
+	if (wlen)
+		*o = '\0';
+
+	return wbuf;
+}
+
+int downloads_in_progress(void)
+{
+	Download *d;
+	int count = 0;
+
+	/* Bit stupid to do it this slow way, can't we maintain a counter? Needs to be accurate though */
+
+	for (d = downloads; d; d = d->next)
+		count++;
+
+	return count;
+}
